/*
 * mingwtlib.c - The MinGW Tools Library
 *
 * Provides low level stub functions often used by GNU tools.
 *  These are usually no-op functions that return a safe value,
 *  usually false. The will eventually may even work for some
 *  platforms - like NT or 2K and not on 9x
 *
 * This file is part of the Mingw32 package.
 *
 * Contributors:
 *  Bruce E. Wampler - original version 17 Feb 2000
 *  date:
 *
 * Copyright 2000, Bruce E. Wampler
 * The license of this software is currently undecided.
 *
 */

#include <mingwtlib.h>

#include <errno.h>
#include <sys/stat.h>	/* for stat/lstat */
#include <signal.h>
#include <process.h>
#include <fcntl.h>
#include <io.h>

/* ============================>>> base_name <<<========================= */
  char* base_name(char const *fullname)
  {
    char const *base = fullname;

    if (!fullname)
	return (char*) 0;
    /* modify standard base_name to allow \ or :
       (as in c:) to be stripped, too */
    while (*fullname)
      {
	if (*fullname == '/' || *fullname == '\\' || *fullname == ':')
	  base = fullname + 1;
	fullname++;
      }
    return (char*) base;
  }

/* ============================>>> euidaccess <<<========================= */
  int euidaccess (const char* path, int mode)
  {
    return 0;
  }

/* ============================>>> lstat <<<========================= */
  int lstat(const char* f, struct stat* s)
  {
     return stat(f,s);
  }

/* ============================>>> chown <<<========================= */
  int chown(const char* dest, uid_t st_uid, gid_t st_gid)
  {
   /* in chown, the short st_gid corresponds to type in struct stat */
     return 0; /* 0 is success */
  }

/* ============================>>> getgroup <<<========================= */
  char* getgroup(int uid)
  {
   static char user[] = "group";
   return user;
  }

/* ============================>>> getuser <<<========================= */
  char* getuser(int uid)
  {
   static char user[] = "user";
   return user;
  }

/* ============================>>> getgid <<<========================= */
  uid_t getgid()
  {
    return 100;
  }

/* ============================>>> getegid <<<========================= */
  uid_t getegid()
  {
    return 100;
  }

/* ============================>>> getuid <<<========================= */
  uid_t getuid()
  {
    return 100;
  }

/* ============================>>> geteuid <<<========================= */
  uid_t geteuid()
  {
    return 100;
  }

/* ============================>>> getgrpid <<<========================= */
  struct group *getgrpid(gid_t gid)
  {
    static struct group g;
    static char* mem[2];
    static char grp[] = "group";
    static char name[] = "user";

    if (gid != getgid())
	return 0;
    mem[0] = name;
    mem[1] = 0;
    g.gr_gid = getgid();
    g.gr_mem = mem;
    g.gr_name = name;
    return &g;
  }

/* ============================>>> getgrnam <<<========================= */
  struct group *getgrnam(const char* name)
  {
    return getgrpid(getgid());
  }

/* ============================>>> getlogin <<<========================= */
  char* getlogin(void)
  {
    static char login[] = "user";
    return login;
  }

/* ============================>>> kill <<<========================= */
  int kill(pid_t pid, int sig)
  {
    if (pid == _getpid() || pid == 0 || pid == -1 || pid == -_getpid())
      {
	if (sig != 0)
	    raise(sig);
        return 0;
      }
    return -1;
  }

/* ============================>>> lchown <<<========================= */
  int lchown(const char* dest, short st_uid, short st_gid)
  {
   /* in lchown, the short st_gid corresponds to type in struct stat */

     errno = ENOSYS;	/* some code relies on the error return */
     return 1; 		/* 1 is failure */
  }

/* ============================>>> link <<<========================= */
  int link(const char* src, const char* dst)
  {
    /* for now, we will always fail. There are some options.
     * we could make a copy of the file under the new name, or
     * make this work just for executables and make a .lnk
     * file.
     */
     errno = ENOSYS;	/* some code relies on the error return */
     return 1; 		/* 1 is failure */
  }

/* ============================>>> mkdir2 <<<========================= */
  int mkdir2(char* dir, int mode)
  {
    struct stat statb;

    if (stat (dir, &statb) == 0)
      {
	errno = EEXIST;		/* stat worked, so it already exists.  */
	return -1;
      }
    /* If stat fails for a reason other than non-existence, return error.  */
    if (errno != ENOENT)
	return -1;

    if (_mkdir(dir) != 0)
      {
	errno = EIO;
	return -1;
      } 
    return chmod(dir,mode);

  }

/* ============================>>> mknod <<<========================= */
  int mknod(const char* node, mode_t m, dev_t d)
  {
    int e  = errno;
    if (S_ISREG(m))
      {
	int fd = open(node, O_CREAT | O_EXCL, S_IWUSR);

	if (fd == -1)
	    return fd;
	close(fd);
	errno = e;
	return 0;
      }
    else if (S_ISCHR(m))
      {
	struct stat statbuf;

	if (stat(node, &statbuf) == 0 &&
	    S_ISCHR(statbuf.st_mode)  &&
	    statbuf.st_dev == d)
	  {
	    errno = EEXIST;
	    return -1;
	  }
      }

      errno = EACCES;
      return -1;
#ifdef STUBONLY
      errno = ENOSYS;	/* some code relies on the error return */
      return 1; 		/* 1 is failure */
#endif
  }

/* =======================>>> mingw_fix_slashes <<<================= */
  void mingw_fix_slashes (char* path)
  {
    /* change all \ to / */
    int ix;

    if (!path)
	return;
    for (ix = 0 ; path[ix] != '\0' ; ++ix)
      {
	if (path[ix] == '\\')
	    path[ix] = '/';
      }
  }

/* =======================>>> nice <<<================= */
  void nice(int id)
  {
    return;
  }

/* =======================>>> readlink <<<============================== */
  int readlink ( const char *path, char *buffer, size_t buf_size)
  {
    errno = ENOENT;
    return (-1);
  }

/* =======================>>> setuid <<<================= */
  void setuid(uid_t id)
  {
    return;
  }

/* =======================>>> strip_trailing_slashes <<<================= */
  void strip_trailing_slashes (char* path)
  {
  /* This is a replacement for the GNU strip_trailing_slashes
   * routine. It takes advantage of the fact that most of the tools
   * call this for every file, and it gives us a nice opportunity
   * to change all the \'s to /'s.
   */

    int ix, lim;

    /* added this code to change all the \'s first */

    if (!path)
	return;
    for (ix = 0 ; path[ix] != '\0' ; ++ix)
      {
	if (path[ix] == '\\')
	    path[ix] = '/';
      }
    --ix;	/* bump down to last character */
    /* now strip all trailing /'s */
    /* need to leave d:/ alone - d: is not the same as d:/ */
    if (path[1] == ':')
	lim = 2;
    else
	lim = 0;
    while (ix > lim && path[ix] == '/')
	path[ix--] = '\0';
  }

/* =======================>>> win32_get_fattr <<<================= */
  int win32_get_fattr(const char* path)
  {
    /* return Win32 file attributes:
     * _A_ARCH - Archive.		(a)
     * _A_HIDDEN - Hidden file.		(h)
     * _A_NORMAL - Normal. Value: 0x00  (-)
     * _A_RDONLY - Read-only.		(r)
     * _A_SUBDIR - Subdirectory.	(d)
     * _A_SYSTEM - System file.		(s)
     *
     *	Shown by Windows explorer in order DRHSA
     */

    int attrs;
    long handle;
    struct _finddata_t fileinfo;

    handle = _findfirst( path, &fileinfo );
    if (handle == -1L)
      {
	return 0;		// just make a normal file
      }

    attrs = fileinfo.attrib;
    _findclose(handle);
    return attrs;
  }

/* =======================>>> win32_get_fattr <<<================= */
  void win32_mode_string(int attrs, char* buff)
  {
    int ix;

    for (ix = 0 ; ix < 5 ; ++ix)	/* set to initially "-----" */
	buff[ix] = '-';
    buff[ix] = 0;
    
    if (attrs & _A_SUBDIR)
	buff[0] = 'D';
    if (attrs & _A_RDONLY)
	buff[1] = 'R';
    if (attrs & _A_HIDDEN)
	buff[2] = 'H';
    if (attrs & _A_SYSTEM)
	buff[3] = 'S';
    if (attrs & _A_ARCH)
	buff[4] = 'A';
  }


